#!/usr/bin/env python
"""
sir_io.py -- BYU sir file I/O routines

 function (image, head, descrip, iaopt)=loadsir('filename')
 function writesir(image, head, descrip, iaopt)
 function printsirhead(head)
 function value=sirheadvalue(option,head)
 function newhead=setsirhead(option,inhead,value)

Created for general python library on 25 Jul 2020

@author: DGL
"""
# revised 10/28/2020 set dimensions to integers in flipud

from __future__ import division
from numpy import fromfile, dtype, double, transpose, ceil, reshape, flipud, fromfile
import numpy as np
import sys

def loadsir(filename,headonly=0):
    """loadsir
    function (image, head, descrip, iaopt)=loadsir('filename' <,headonly=1>)
    - or -
    function SIR=loadsir('filename' <,headonly=1>)

    load a sirf image and head array into python.  Also sets global variables
    containing the image transformation information from file header

    image:      image array
    head:       scaled header information block
    descrip:    second header description string
    iaopt:      second header integer array
    filename:   name of sir input file
    headonly:   (optional)when present and equal to one, does not read image, just head
    """
    
    fid = open(filename)
    data_types = dtype('int16').newbyteorder('>')
    data_typec = dtype('int8').newbyteorder('>')
    data_typef = dtype('float32').newbyteorder('>')
    head = double(fromfile(fid,dtype=data_types,count=256,sep=""))  # read header
    #head = double(fromfile(filename,dtype=data_types,count=256,sep=""))  # read header
    
    nhtype = head[4]
    if nhtype < 20:
        nhtype = 1.0
        head[4] = 1.0

    nhead = head[40]
    if nhtype == 1:
        nhead = 1.0
        head[40] = 1.0
        head[41] = 0.0
        head[42] = 0.0
        head[43] = 0.0

    ndes      = head[41]
    ldes      = head[42]
    nia       = head[43]
    idatatype = head[47]
    iopt      = head[16]    # transformation option

    if nhtype < 30:  # old header format
    # set version 3.0 parameters to header version 2.0 defaults     
        if iopt==-1:  # image only
            ideg_sc = 10.0
            iscale_sc = 1000.0
            i0_sc = 100.0
            ixdeg_off = 0.0
            iydeg_off = 0.0
            ia0_off = 0.0
            ib0_off = 0.0
        elif iopt==0: # rectalinear lat/lon
            ideg_sc = 100.0
            iscale_sc = 1000.0
            i0_sc = 100.0
            ixdeg_off = -100.0
            iydeg_off = 0.0
            ia0_off = 0.0
            ib0_off = 0.0
        elif (iopt==1) or (iopt==2): # lambert
            ideg_sc = 100.0
            iscale_sc = 1000.0
            i0_sc = 1.0
            ixdeg_off = 0.0
            iydeg_off = 0.0
            ia0_off = 0.0
            ib0_off = 0.0
        elif iopt==5: # polar stereographic
            ideg_sc = 100.0
            iscale_sc = 100.0
            i0_sc = 1.0
            ixdeg_off = -100.0
            iydeg_off = 0.0
            ia0_off = 0.0
            ib0_off = 0.0
        elif (iopt==8) or (iopt==9) or (iopt==10): # EASE2 grid
            ideg_sc = 10.0
            iscale_sc = 1000.0
            i0_sc = 1.0
            ixdeg_off = 0.0
            iydeg_off = 0.0
            ia0_off = 0.0
            ib0_off = 0.0
        elif (iopt==11) or (iopt==12) or (iopt==13): # EASE grid
            ideg_sc = 10.0
            iscale_sc = 1000.0
            i0_sc = 10.0
            ixdeg_off = 0.0
            iydeg_off = 0.0
            ia0_off = 0.0
            ib0_off = 0.0
        else: #  unknown default scaling
            ideg_sc = 100.0
            iscale_sc = 1000.0
            i0_sc = 100.0
            ixdeg_off = 0.0
            iydeg_off = 0.0
            ia0_off = 0.0
            ib0_off = 0.0

        head[ 39] = iscale_sc
        head[126] = ixdeg_off
        head[127] = iydeg_off
        head[168] = ideg_sc
        head[189] = ia0_off
        head[240] = ib0_off
        head[255] = i0_sc 
    else:  # get projection parameters offset and scale factors
        iscale_sc = head[ 39]
        ixdeg_off = head[126]
        iydeg_off = head[127]
        ideg_sc   = head[168]
        ia0_off   = head[189]
        ib0_off   = head[240]
        i0_sc     = head[255]

    #    decode projection transformation 
    xdeg   = head[2]/ideg_sc - ixdeg_off
    ydeg   = head[3]/ideg_sc - iydeg_off
    ascale = head[5]/iscale_sc
    bscale = head[6]/iscale_sc
    a0     = head[7]/i0_sc - ia0_off
    b0     = head[8]/i0_sc - ib0_off
    #    get special cases which depend on transformation option
    if iopt==-1:   # image only
        pass
    elif iopt==0:  # rectalinear lat/lon
        pass
    elif (iopt==1) or (iopt==2): # lambert
        ascale = iscale_sc/head[5]
        bscale = iscale_sc/head[6]
    elif iopt==5: # polar stereographic
        pass
    elif (iopt==8) or (iopt==9) or (iopt==10): # EASE2 grid
        pass
    elif (iopt==11) or (iopt==12) or (iopt==13): # EASE grid
        ascale = 2.0*(head[5]/iscale_sc)*6371.228/25.067525
        bscale = 2.0*(head[6]/iscale_sc)*25.067525
    else: # unknown default scaling
        print ("*** Unrecognized SIR option in loadsir ***")

    head[2] = xdeg
    head[3] = ydeg
    head[5] = ascale
    head[6] = bscale
    head[7] = a0
    head[8] = b0

    if head[10]==0:  # iscale
        head[10] = 1.0
        
    s = 1.0/head[10]
    soff = 32767.0/head[10]
    if idatatype==1:
        soff = 128.0/head[10]

    ioff = head[9]
    anodata = head[48]*s + ioff + soff
    vmin = head[49]*s + ioff + soff
    vmax = head[50]*s + ioff + soff

    if idatatype==4:  # floating point file
        # open file again for separately reading floats
        fid2 = open(filename)
        # skip beginning of head
        fromfile(fid2,dtype=data_types,count=51,sep="")
        # read floating point values from file
        fl = double(fromfile(fid2,dtype=data_typef,count=3,sep=""))
        fid2.close()
        anodata = fl[0]
        vmin = fl[1]
        vmax = fl[2]

    head[45] = head[45]*0.1
    head[48] = anodata
    head[49] = vmin
    head[50] = vmax

    descrip = []
    iaopt = []

    if nhead>1:
        if ndes>0:
            descrip = double(fromfile(fid,dtype=data_typec,count=ndes*512,sep=""))
            descrip = transpose(descrip[1:ldes])
            m,n = descrip.shape
            for j in range(1,n/2+1):
                k = (j-1)*2 + 1
                t = descrip[k-1]
                descrip[k-1] = descrip[k]
                descrip[k] = t        
        if nia>0:
            nia1 = 256.0*ceil(nia/256)
            iaopt = double(fromfile(fid,dtype=data_types,count=nia1,sep=""))
            iaopt = transpose(iaopt[1:nia])
# read image data
  
    if headonly!=1:
      if idatatype==1: # very rare
        # disp(['Read byte data: ' num2str(head(1)) ' x ' num2str(head(2))]);
        im_in = double(fromfile(fid,dtype=data_typec,count=int(head[0]*head[1]),sep=""))  # read byte image data
        image = flipud(reshape(s*im_in+soff+ioff,(int(head[1]),int(head[0])),order='C'))   # scale data to floating point and
        # change origin location 
      elif idatatype==4: # rare
        # disp(['Read float data: ' num2str(head(1)) ' x ' num2str(head(2))]);
        im_in = double(fromfile(fid,dtype=data_typef,count=int(head[0]*head[1]),sep=""))
        image = flipud(reshape(im_in,(int(head[1]),int(head[0])),order='C'))  # read floating point data
      else:  # most commonly used
        # disp(['Read integer data: ' num2str(head(1)) ' x ' num2str(head(2))]);
        im_in = double(fromfile(fid,dtype=data_types,count=int(head[0]*head[1]),sep=""))  # read integer image data
        image = flipud(reshape(s*im_in+soff+ioff,(int(head[1]),int(head[0])),order='C'))   # scale data to floating point and
        # change origin location for display
    else:
        image=None

    if nhtype==1:                     # if old-style header, set default values
        vmin = min(image.flatten(1))
        vmax = max(image.flatten(1))
        anodata = vmin
        head[48] = anodata
        head[49] = vmin
        head[50] = vmax
        if vmin==-32:
            head[18] = 1.0
        elif vmin==-3.2:
            head[18] = 2.0
            
        head[44] = 2.0
        head[45] = 53.0

    fid.close()
    return image, head, descrip, iaopt


def writesir(filename, image, head):
    """writesir
    function writesir(image, head, descrip, iaopt)

    write a sirf image to a file with information from head array
    which must version 30 or higher. does not write extra header info
     * Note: there is no error checking of the head array, which must
             be properly formed and defined -- suggest copying and 
             modifying a valid header

    filename:   name of output file
    image:      image array
    head:       scaled header information block
    """
    # open output file for writing
    fid = open(filename,'w')

    #head[47]=4 # test writing float

    #ndes      = head[41]
    #ldes      = head[42]
    #nia       = head[43]
    idatatype = head[47]
    iopt      = head[16]    # transformation option

    head[40]=1 # override multiple header options
    head[41]=0
    head[42]=0
    head[43]=0

    if head[10]==0:  # iscale
        head[10] = 1.0

    # convert header to 256 int16s and write to file
    # TODO: net set byte order when writing

    # copy over base values
    record=np.int16(head)

    # special cases to prepare record for writing
    record[45] = int(head[45]*10) # ipol

    # now apply scalings of float values 
    # get projection parameters offset and scale factors
    iscale_sc = head[ 39]
    ixdeg_off = head[126]
    iydeg_off = head[127]
    ideg_sc   = head[168]
    ia0_off   = head[189]
    ib0_off   = head[240]
    i0_sc     = head[255]

    # default encode projection transformation 
    record[2]=int((head[2]+ixdeg_off)*ideg_sc) # xdeg
    record[3]=int((head[3]+iydeg_off)*ideg_sc) # ydeg
    record[5]=int(head[5]*iscale_sc) # ascale
    record[6]=int(head[6]*iscale_sc) # bscale
    record[7]=int((head[7]+ia0_off)*i0_sc) # a0
    record[8]=int((head[8]+ia0_off)*i0_sc) # b0

    # special cases which depend on transformation option
    if (iopt==1) or (iopt==2): # lambert
        record[5]=int(iscale_sc/head[5]) # ascale
        record[6]=int(iscale_sc/head[6]) # bscale
    elif (iopt==8) or (iopt==9) or (iopt==10): # EASE2 grid
        record[5]=int(head[5]) # ascale
        record[6]=int(head[6]) # bscale
    elif (iopt==11) or (iopt==12) or (iopt==13): # EASE grid
        record[5]=int(head[5]*iscale_sc*25.067525/(2.*6371.228)) # ascale
        record[6]=int(head[6]*iscale_sc*25.067525/2.) # bscale
    else: # unknown default scaling
        print ("*** Unrecognized SIR option in writesir ***")

    # Notes:
    # if idatatype = 1 data is stored as bytes and minv=128
    # if idatatype = 2 data is stored as 2 byte integers and minv=32766
    # if idatatype = 4 data is stored as IEEE floating point
    # anodata, vmin, and vmax are stored as ints or floats

    s = head[10]
    soff = 32767.0/head[10]
    if idatatype==1:
        soff = 128.0/head[10]

    # integer storage of anodata, vmin, and vmax
    ioff = head[9]
    record[48]=int((head[48]-soff-ioff)*s) # anondata
    record[49]=int((head[49]-soff-ioff)*s) # vmin
    record[50]=int((head[50]-soff-ioff)*s) # vmax

    # write header record in chunks to enable including float values
    if sys.byteorder=='little':  # little-endian machine
        np.array(record[0:51],dtype='int16').byteswap().tofile(fid)
        # write floating point values
        np.array(head[48:51],dtype='float32').byteswap().tofile(fid)
        np.array(record[57:256],dtype='int16').byteswap().tofile(fid)
    else:                        # big-endian machine
        np.array(record[0:51],dtype='int16').tofile(fid)
        # write floating point values
        np.array(head[48:51],dtype='float32').tofile(fid)
        np.array(record[57:256],dtype='int16').tofile(fid)

    # write image data
    if idatatype==1:   # 1 byte format images (very rare)
        temp=np.flipud(np.int8((image-soff-ioff)*s))
        np.array(temp,dtype='int8').tofile(fid)
    elif idatatype==4: # 4 byte floating point data
        if sys.byteorder=='little':
            np.array(np.flipud(image),dtype='float32').byteswap().tofile(fid)
        else:
            np.array(np.flipud(image),dtype='float32').tofile(fid)
    else:              # 2 byte ints -- most commonly used
        temp=np.flipud(np.int16((image-soff-ioff)*s))
        if sys.byteorder=='little':
            np.array(temp.byteswap(),dtype='int16').tofile(fid)
        else:
            np.array(temp,dtype='int16').tofile(fid)

    # check to make sure file is a multiple of 512 bytes long
    bytes=int(np.shape(image)[0]*np.shape(image)[1]*idatatype) % 512
    if bytes > 0:
        np.array(np.zeros(512-bytes),dtype='int8').tofile(fid)

    # close output file
    fid.close()
    return

######
# internally used routines
#
def sirstring(a,cnt):
    """sirstring(a,cnt)
    decodes and returns a string starting at a that is cnt bytes long
    """
    string=np.zeros(2*cnt,dtype='i1')
    for i in range(cnt):
        j=(i-1)*2+1;
        string[j-1]=(np.mod(a[i-1],256))
        string[j]=np.floor(a[i-1]/256)
    res=(str(bytearray(string).decode("utf-8")).rstrip(' \n\0\1\r'))

    return res
#
def encodesirstring(string,cnt):
    """encodesirstring(input_string,cnt)
    encodes a char string cnt bytes long into a SIR header int2 array elements
    """
    bb=np.zeros(2*cnt,dtype='i1')
    for i in range(0,len(string)):
        bb[i]=ord(string[i])
    out=np.zeros(cnt,dtype='i32')
    for i in range(0,cnt):
        j=(i-1)*2+1
        out[i]=bb[j]+256*bb[j+1]

    return out
#
#####



#####
# printsirhead
####    
def printsirhead(head):
    """printsirhead(head)

    prints information from the SIR file header info array from loadsir

    INPUTS:
        head:       scaled header information block
    """
    nhtype = np.int(head[4])
    nsx    = np.int(head[0])
    nsy    = np.int(head[1])
    iopt   = np.int(head[16])
    xdeg   = head[2]
    ydeg   = head[3]
    ascale = head[5]
    bscale = head[6]
    a0     = head[7]
    b0     = head[8]

    ioff=np.int(head[9])
    iscale=np.int(head[10])
    nhead = np.int(head[40])
    if nhtype == 1:
        nhead = 1

    ndes = np.int(head[41])
    ldes = np.int(head[42])
    nia = np.int(head[43])
    idatatype=np.int(head[47])
    
    iscale_sc = np.int(head[30])
    ixdeg_off = np.int(head[126])
    iydeg_off = np.int(head[127])
    ideg_sc   = np.int(head[168])
    ia0_off   = np.int(head[189])
    ib0_off   = np.int(head[240])
    i0_sc     = np.int(head[255])

    if iopt==-1:                # image only
        print ("Image only form (%d)" % iopt)
        print ("  x,y scale:   %f %f  %d" % (ascale, bscale, iscale_sc))
        print ("  x,y offsets: %f %f  %d %d %d" % (a0, b0, ia0_off, ib0_off, i0_sc))
        print ("  x,y span:    %f %f  %d %d %d" % (xdeg, ydeg, ixdeg_off, iydeg_off, ideg_sc))

    elif iopt==0:               # rectalinear lat/lon
        print ("Lat/Long Rectangular form (%d) ",iopt)
        print ("  x,y scale: (pix/deg) %f %f  %d" % (ascale, bscale, iscale_sc))
        print ("  x,y offsets: (deg)   %f %f  %d %d %d" % (a0, b0, ia0_off, ib0_off, i0_sc))
        print ("  x,y span: (deg)      %f %f  %d %d %d" % (xdeg, ydeg, ixdeg_off, iydeg_off, ideg_sc))

    elif iopt==1 or iopt==2: # lambert
        if iopt == 1:
            print ("Lambert form (global radius) (%d)" % iopt)
        else:
            print ("Lambert form (local radius) (%d)" % iopt)

        iascale =1/ascale
        ibscale =1/bscale
        print ("  x,y scale: (km/pix) %f %f  %d" % (iascale, ibscale, iscale_sc))
        print ("  x,y corner: (km)    %f %f  %d %d %d" % (a0, b0, ia0_off, ib0_off, i0_sc))
        print ("  x,y span: (deg)     %f %f  %d %d %d " % (xdeg, ydeg, ixdeg_off, iydeg_off, ideg_sc))

    elif iopt==5:               # polar stereographic
        print ("Polar Stereographic form (%d)" % iopt)
        print ("  x,y scale: (km/pix) %f %f  %d" % (ascale, bscale, iscale_sc))
        print ("  x,y offsets: (km)   %f %f  %d %d %d" % (a0, b0, ia0_off, ib0_off, i0_sc))
        print ("  Center: (lon,lat)   %f %f  %d %d %d" % (xdeg, ydeg, ixdeg_off, iydeg_off, ideg_sc))

    elif (iopt==8) or (iopt==9) or (iopt==10):  # EASE2
        if iopt==8:
            print ("EASE2 form Northern Hemisphere(%d)" % iopt)
        elif iopt==9:
            print ("EASE2 form Southern Hemisphere (%d)" % iopt)
        else:
            print ("EASE2 form Global (%d)" % iopt)
        (map_equatorial_radius_m, map_eccentricity, \
             e2, map_reference_latitude, map_reference_longitude, \
             map_second_reference_latitude, sin_phi1, cos_phi1, kz, \
             map_scale, bcols, brows, r0, s0, epsilon) = ease2_map_info(iopt, np.int(ascale), np.int(bscale))
        print ("  EASE2 scale: (%d %d) %f km/pix" % (ascale, bscale, map_scale*0.001))
        print ("  Pixel origin: (c,r) %d %d   %d %d %d %d" % (a0, b0, ia0_off, ib0_off, i0_sc, iscale_sc))
        print ("  Center pixel: (c,r) %d %d   %d %d %d" % (xdeg, ydeg, ixdeg_off, iydeg_off, ideg_sc))

    elif (iopt==11) or (iopt==12) or (iopt==13):  # EASE
        if iopt==11:
            print ("EASE form Northern Hemisphere(%d)" % iopt)
        elif iopt==12:
            print ("EASE form Southern Hemisphere (%d)" % iopt)
        else:
            print ("EASE form Global (%d)" % iopt)
        print ("  Scale: (c,r)  %d %d  %d" % (ascale, ascale, iscale_sc))
        print ("  Origin: (c,r) %d %d  %d %d %d" % (a0, b0, ia0_off, ib0_off, i0_sc))
        print ("  Center: (c,r) %d %d  %d %d %d" % (xdeg, ydeg, ixdeg_off, iydeg_off, ideg_sc))

    else:
        print ("*** Unknown SIR transformation in file header: %d" % iopt)
    

    print ("  Pixels:  %d by %d " % (nsx, nsy))
    print ("  Year: %d  Start Day: %d   Min:  %d  End Day: %d  Min: %d" % (np.int(head[12]), np.int(head[11]), np.int(head[13]), np.int(head[14]), np.int(head[16])))
    print ("  Region: %d  Type: %d  Htype: %d  Nhead: %d" % (np.int(head[17]), np.int(head[18]), np.int(head[4]), nhead))
    print ("  Nodata value: %f  Vmin: %f  Vmax: %f" % (head[48], head[49], head[50]))
    if idatatype == 4 :
        print ("  Datatype: float")
    elif idatatype == 1 :
        print ("  Datatype: byte  offset: %d  scale: %d" % (ioff, iscale))
    else:
        print ("  Datatype: short  offset: %d  scale: %d" % (ioff, iscale))
    print ("  Frequency: %d   Pol: %d" % (head[45], np.int(head[44])))
    print ("  Type: %s" % sirstring(head[57:77],20))
    print ("  Sensor: %s" % sirstring(head[19:39],20))
    print ("  Title: %s" % sirstring(head[128:168],40))
    print ("  Tag: %s" % sirstring(head[169:189],20))
    print ("  Creator: %s" % sirstring(head[190:240],50))
    print ("  Created: %s" % sirstring(head[241:255],14))

#####
# sirheadvalue
####
def sirheadvalue(option,head):
    """ value=sirheadvalue(option,head)

  extracts information from the SIR file header info array

  INPUTS:
        option:     item to return
        head:       scaled header information block

  Option list:  (these return numeric values)
  'nsx'   'nsy'     'xdeg'    'ydeg'    'nhtype'    'ascale'  'bscale' 'a0'
  'b0'    'ioff'    'iscale'  'iyear'   'isday'     'ismin'   'ieday' 'iemin'
  'iopt'  'iregion' 'itype'   'nhead'   'ndes'      'ldes'     'nia'
  'vmin'  'vmax'    'anodata' 'ispare1' 'idatatype' 'i0_sc'
  'iscale_sc'     'ixdeg_off' 'iydeg_off'            'ideg_sc'  
  'ia0_off'       'ib0_off'    'ifreqm' 'ipol'

  (these options return string values)
  'sensor' 'title' 'type' 'tag' 'crproc' 'crtime'

   see also:
   setsirhead (sets a sir header value by name)
   display_head (returns list of head options and values)

    """
    if option.lower()=='nsx': # ints and floats
        value=int(head[0])
    elif option.lower()=='nsy':
        value=int(head[1])
    elif option.lower()=='xdeg':
        value=head[2]
    elif option.lower()=='ydeg':
        value=head[3]
    elif option.lower()=='nhtype':
        value=int(head[4])
    elif option.lower()=='ascale':
        value=head[5]
    elif option.lower()=='bscale':
        value=head[6]
    elif option.lower()=='a0':
        value=head[7]
    elif option.lower()=='b0':
        value=head[8]
    elif option.lower()=='ioff':
        value=int(head[9])
    elif option.lower()=='iscale':
        value=int(head[10])
    elif option.lower()=='iyear':
        value=int(head[11])
    elif option.lower()=='isday':
        value=int(head[12])
    elif option.lower()=='ismin':
        value=int(head[13])
    elif option.lower()=='ieday':
        value=int(head[14])
    elif option.lower()=='iemin':
        value=int(head[15])
    elif option.lower()=='iopt':
        value=int(head[16])
    elif option.lower()=='iregion':
        value=int(head[17])
    elif option.lower()=='itype':
        value=int(head[18])
    elif option.lower()=='iscale_sc':
        value=int(head[39])
    elif option.lower()=='nhead':
        value=int(head[40])
    elif option.lower()=='ndes':
        value=int(head[41])
    elif option.lower()=='ldes':
        value=int(head[42])
    elif option.lower()=='nia':
        value=int(head[43])
    elif option.lower()=='ipol':
        value=int(head[44])
    elif option.lower()=='ifreqm':
        value=int(head[45])
    elif option.lower()=='ispare1':
        value=int(head[46])
    elif option.lower()=='idatatype':
        value=int(head[47])
    elif option.lower()=='anodata':
        value=head[48]
    elif option.lower()=='vmin':
        value=head[49]
    elif option.lower()=='vmax':
        value=head[50]
    elif option.lower()=='ixdeg_off':
        value=int(head[126])
    elif option.lower()=='iydeg_off':
        value=int(head[127])
    elif option.lower()=='ideg_sc':
        value=int(head[168])
    elif option.lower()=='ib0_of':
        value=int(head[240])
    elif option.lower()=='i0_sc':
        value=int(head[255])
        
    elif option.lower()=='sensor':  # strings
        value=sirstring(head[19:39],20)
    elif option.lower()=='title':
        value=sirstring(head[128:168],40)
    elif option.lower()=='type':
        value=sirstring(head[57:77],20)
    elif option.lower()=='tag':
        value=sirstring(head[169:189],20)
    elif option.lower()=='crproc':
        value=sirstring(head[190:240],50)
    elif option.lower()=='crtime':
        value=sirstring(head[241:255],14)
    else:  
        print('*** Unknown option in sirheadvalue %s',option);

    return value


#####
# setsirhead
####
def setsirhead(option,inhead,value):
    """ newhead=setsirhead(option,inhead,value)

  copies inhead to outhead changing value of option to value
    *use with great caution: there is no error checking!
     it is possible to set head values to invalid values that will 
     not work properly.

  INPUTS:
        option:     item to set
        head:       scaled header information block
        value:      new value

  Option list:  (numeric values)
  'nsx'   'nsy'     'xdeg'    'ydeg'    'nhtype'    'ascale'  'bscale' 'a0'
  'b0'    'ioff'    'iscale'  'iyear'   'isday'     'ismin'   'ieday' 'iemin'
  'iopt'  'iregion' 'itype'   'nhead'   'ndes'      'ldes'     'nia'
  'vmin'  'vmax'    'anodata' 'ispare1' 'idatatype' 'i0_sc'
  'iscale_sc'     'ixdeg_off' 'iydeg_off'            'ideg_sc'  
  'ia0_off'       'ib0_off'    'ifreqm' 'ipol'

  (string values)
  'sensor' 'title' 'type' 'tag' 'crproc' 'crtime'

   see also:
   sirheadvalue (returns a sir header value by name)
   display_head (returns list of head options and values)

    """
    head=np.copy(inhead);
    if option.lower()=='nsx': # ints and floats
        head[0]=int(value)
    elif option.lower()=='nsy':
        head[1]=int(value)
    elif option.lower()=='xdeg':
        head[2]=value
    elif option.lower()=='ydeg':
        head[3]=value
    elif option.lower()=='nhtype':
        head[4]=int(value)
    elif option.lower()=='ascale':
        head[5]=value
    elif option.lower()=='bscale':
        head[6]=value
    elif option.lower()=='a0':
        head[7]=value
    elif option.lower()=='b0':
        head[8]=value
    elif option.lower()=='ioff':
        head[9]=int(value)
    elif option.lower()=='iscale':
        head[10]=int(value)
    elif option.lower()=='iyear':
        head[11]=int(value)
    elif option.lower()=='isday':
        head[12]=int(value)
    elif option.lower()=='ismin':
        head[13]=int(value)
    elif option.lower()=='ieday':
        head[14]=int(value)
    elif option.lower()=='iemin':
        head[15]=int(value)
    elif option.lower()=='iopt':
        head[16]=int(value)
    elif option.lower()=='iregion':
        head[17]=int(value)
    elif option.lower()=='itype':
        head[18]=int(value)
    elif option.lower()=='iscale_sc':
        head[39]=int(value)
    elif option.lower()=='nhead':
        head[40]=int(value)
    elif option.lower()=='ndes':
        head[41]=int(value)
    elif option.lower()=='ldes':
        head[42]=int(value)
    elif option.lower()=='nia':
        head[43]=int(value)
    elif option.lower()=='ipol':
        head[44]=int(value)
    elif option.lower()=='ifreqm':
        head[45]=int(value)
    elif option.lower()=='ispare1':
        head[46]=int(value)
    elif option.lower()=='idatatype':
        head[47]=int(value)
    elif option.lower()=='anodata':
        head[48]=value
    elif option.lower()=='vmin':
        head[49]=value
    elif option.lower()=='vmax':
        head[50]=value
    elif option.lower()=='ixdeg_off':
        head[126]=int(value)
    elif option.lower()=='iydeg_off':
        head[168]=int(value)
    elif option.lower()=='ib0_of':
        head[240]=int(value)
    elif option.lower()=='i0_sc':
        head[255]=int(value)
        
    elif option.lower()=='sensor':  # strings
        head[19:39]=encodesirstring(value,20)
    elif option.lower()=='title':
        head[128:168]=encodesirstring(value,40)
    elif option.lower()=='type':
        head[57:77]=encodesirstring(value,20)
    elif option.lower()=='tag':
        head[169:189]=encodesirstring(value,20)
    elif option.lower()=='crproc':
        head[190:240]=encodesirstring(value,50)
    elif option.lower()=='crtime':
        head[241:255]=encodesirstring(value,14)
    else:  
        print('*** Unknown option in setsirhead %s',option);

    return head

#########
# default operation: load file
#########
def main(sir_fname):
    # A test driver
    sir = loadsir(sir_fname)
    return sir

if __name__ == "__main__":
    main(sys.argv[1])



