#!/usr/bin/env python
"""
Created 24 Jul 2020
This script enables resizing of a BYU .sir file by an integer amount, either
larger or smaller. Note: EASE2 images require a power of 2 resize factor

@author: DGL at BYU
"""

######
# Imports
######
from loadsir import loadsir
from sirhead import *
from writesir import writesir


######
# Driver routine to resize SIR image files
######
def sirfile_resize(N,fname,fout):
    """ 
    sirresize(N,fname_in,fname_out)

    N:         integer resize factor (N<0 is shrink, N>1 is increase)
    fname_in:  input SIR file name
    fname_out: output SIR file name


    """
    # Load in the SIR image
    sir = loadsir(fname)
    # sir[0] is the image array; sir[1] is the header; sir[2]=iaopt; sir[3]=descript

    # process input file
    img=sir[0]
    head=sir[1]
    anodata=-32

    # generate resized image
    ret=sirresize_image(N,img,anodata)
    out=ret[0]
    N1=ret[1]

    # generate resized SIR file header
    ret=sirresize_head(N1,head)
    ohead=ret[0]

    # write output file
    writesir(fout,out,ohead)


######
# Routine to resize SIR image arrays
######
def sirresize_image(N,img,anodata):
    """ 
    (outimg, N1)= sirresize_image(N,img,anodata)

    N:       integer resize factor (N<1 is shrink, N>1 is increase)
    img:     input image array
    anodata: minimum threshold to include in averaged data (only used for shrinking array)

    outimg:  output image array
    N1:      actual integer N value used

    """
    # make sure N is an integer
    N1=int(float(N))

    if N1==0 or N1==-1 or N1==1: # no change
        out=img
    elif N1 > 0: # pixel replicate to up size image
        dims=(N1*img.shape[0],N1*img.shape[1])
        out=np.zeros((dims[0],dims[1]))
        for i in range(0,img.shape[0]):
            for j in range(0,img.shape[1]):
                for x in range(0,N1):
                    for y in range(0,N1):
                        out[i*N1+x,j*N1+y]=img[i,j]
    else: # average down resolution to down size image
        N2=-N1
        dims=(int(img.shape[0]/N2),int(img.shape[1]/N2))
        if dims[0]<1:
            dims=(1,dims[1])
        if dims[1]<1:
            dims=(dims[0],1)
        out=np.zeros(dims)
        for i in range(0,dims[0]):
            for j in range(0,dims[1]):
                sum=0.
                cnt=0
                for x in range(0,N2):
                    for y in range(0,N2):
                        if img[i*N2+x,j*N2+y] > anodata+0.00001:
                            cnt=cnt+1
                            sum=sum+img[i*N2+x,j*N2+y]
                if cnt > 0:
                    out[i,j]=sum/cnt
                else:
                    out[i,j]=anodata
    return (out, N1)


######
# Routine to resize SIR head array
######
def sirresize_head(N1,hin):
    """ 
    (headout,N1)=sirresize_image(N,headin)

    N:       integer resize factor (N<1 is shrink, N>1 is increase)
    head:    SIR file head array

    headout: output SIR file head array
    N1:      actual integer N value used
   
    """
    # make sure N is an integer
    N=int(float(N1))

    if N==0 or N==-1 or N==1: # no change
        hout=hin;
    else:
        if N<1:
            N=-1.0/N
        # useful shortcuts
        nsx    = hin[0]
        nsy    = hin[1]
        xdeg   = hin[2]
        ydeg   = hin[3]
        ascale = hin[5]
        bscale = hin[6]
        a0     = hin[7]
        b0     = hin[8]
        iopt   = hin[16]
  
        # output size
        nsx = int(N * nsx)
        nsy = int(N * nsy)

        # set header values, first line creates output header and copies contents
        hout=setsirhead('nsx',hin,nsx)
        hout=setsirhead('nsy',hout,nsy)

        if iopt==-1: # image only
            xdeg = N * xdeg
            ydeg = N * ydeg
            hout=setsirhead('xdeg',hout,xdeg)  
            hout=setsirhead('ydeg',hout,ydeg)  
        elif iopt==0:   # lat/lon
            xdeg = N * xdeg
            ydeg = N * ydeg
            hout=setsirhead('xdeg',hout,xdeg)  
            hout=setsirhead('ydeg',hout,ydeg)  
        elif iopt==1:   # Lambert
            ascale = N * ascale 
            bscale = N * bscale 
            hout=setsirhead('ascale',hout,ascale)  
            hout=setsirhead('bscale',hout,bscale)  
        elif iopt==2:   # Lambert
            ascale = N * ascale 
            bscale = N * bscale 
            hout=setsirhead('ascale',hout,ascale)  
            hout=setsirhead('bscale',hout,bscale)  
        elif iopt==5:   # polar stereographic
            ascale = ascale / N 
            bscale = bscale / N 
            hout=setsirhead('ascale',hout,ascale)  
            hout=setsirhead('bscale',hout,bscale)  
        elif iopt==8:   # EASE2 grid
            n=int(np.log(N)/np.log(2))
            ascale = ascale + n
            hout=setsirhead('ascale',hout,ascale)
        elif iopt==9:   # EASE2 grid
            n=int(np.log(N)/np.log(2))
            ascale = ascale + n
            hout=setsirhead('ascale',hout,ascale)
        elif iopt==10:  # EASE2 grid
            n=int(np.log(N)/np.log(2))
            ascale = ascale + n
            hout=setsirhead('ascale',hout,ascale)
        elif iopt==11:  # EASE1 grid
            n=int(np.log(N)/np.log(2))
            ascale = ascale / N
            bscale = N * bscale
            hout=setsirhead('ascale',hout,ascale)
            hout=setsirhead('bscale',hout,bscale)
        elif iopt==12:  # EASE1 grid
            n=int(np.log(N)/np.log(2))
            ascale = ascale / N
            bscale = N * bscale
            hout=setsirhead('ascale',hout,ascale)
            hout=setsirhead('bscale',hout,bscale)
        elif iopt==13:  # EASE1 grid
            n=int(np.log(N)/np.log(2))
            ascale = ascale / N
            bscale = N * bscale
            hout=setsirhead('ascale',hout,ascale)
            hout=setsirhead('bscale',hout,bscale)

    return (hout, N1)

def main(argv): # pass in resize, input, and output file names arguments
    if len(argv)>1:
        N = argv[1]
        if len(argv)>2:
            fname = argv[2]
            if len(argv)>3:
                fout=argv[3]
            else:
                fout = fname + '.grd'
        else:
            print('Input file name required')
    else: # no input arguments
        print('sirresize: resizes SIR files')
        print('  sirresize N fname_in fname_out')
        print('    N:         integer resize factor (N<0 is shrink, N>1 is increase)')
        print('    fname_in:  input SIR file name')
        print('    fname_out: output SIR file name')
        print(' ')

        if 1: # test case
            N=-10  # shrink
            fname = 'greeni.sir'
            fout = fname + '.grd'

    # call work routine
    sirfile_resize(N,fname,fout)

    print('Resize %s by %s to file %s' % (fname,N,fout))
    if 0: # show before/after headers
        printsirhead(head)
        printsirhead(ohead)

    if 0: # display image
        if 0: # show original
            plt.figure()
            plt.imshow(img, cmap=plt.cm.gray, interpolation='nearest', aspect='equal')
            # add axes and labels
            plt.axis('image')
            plt.title(fname)
            plt.xlabel('x')
            plt.ylabel('y')
            # add a colobar
            plt.colorbar()

        # show resized image
        plt.figure()
        plt.imshow(out, cmap=plt.cm.gray, interpolation='nearest', aspect='equal')
        # add axes and labels
        plt.axis('image')
        plt.title(fout)
        plt.xlabel('x')
        plt.ylabel('y')
        # add a colobar
        plt.colorbar()

        plt.show()

if __name__ == '__main__':
    main(sys.argv)
